//
//  ViewController.swift
//  Projet - RottenTomatoes version swift
//
//  Created by Alain on 2014-09-13.
//  Copyright (c) 2014 Alain. All rights reserved.
//
//  ============================================================================================
//  À l'usage exclusif des étudiants et étudiantes de
//  Techniques d'Intégration Multimédia
//  du cégep de Saint-Jérôme.
//  --------------------------------------------------------------------------------------------
//  Il est interdit de reproduire, en tout ou en partie, à des fins commerciales,
//  le code source, les scènes, les éléments graphiques, les classes et
//  tout autre contenu du présent projet sans l’autorisation écrite de l'auteur.
//
//  Pour obtenir l’autorisation de reproduire ou d’utiliser, en tout ou en partie,
//  le présent projet, veuillez communiquer avec:
//
//  Alain Boudreault, aboudrea@cstj.qc.ca
//
//  ============================================================================================
//  Version de départ

import UIKit

class ViewController: UIViewController, UITextFieldDelegate, UICollectionViewDataSource {

    @IBOutlet weak var collectionDeFilms:UICollectionView!
    @IBOutlet weak var progression:UIActivityIndicatorView!
    @IBOutlet weak var ViewVideos:UICollectionView!
    @IBOutlet weak var nbResultat:UILabel!
    
    // ***  Les constantes de la classe
    let NB_FILMS = "50"
    let CLE_API  = Cle().CLE_API

    // *** Les propriétés de la classe
    var tableauDesFilms = NSDictionary()
    
    // *** Les méthodes de la classe
    // *************************************************************************************************
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        rechercherFilms("Star")
        
        //let titreDuPremierFilm = (((tableauDesFilms["movies"] as NSArray)[0] as NSDictionary)["title"]!) as NSString
        
        //println(titreDuPremierFilm)
        
    } // viewDidLoad

    // *************************************************************************************************
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
        println("didReceiveMemoryWarning")
    } // didReceiveMemoryWarning

    
    // Passer les info à la scène de détail
    // *************************************************************************************************
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        var vc = segue.destinationViewController as DetailFilmViewController

        // Passer les informations de la sélection courante à la scène de détail
        let selectionCourante = collectionDeFilms.indexPathForCell(sender as UICollectionViewCell)?.row
        vc.detailFilm = (tableauDesFilms["movies"] as NSArray)[selectionCourante!] as NSDictionary
        
    } // // Passer les info à la scène de détail
    
    
    /// *****************************
    /// Les méthodes de protocoles:
    /// delegate et dataSource
    
    
    // *************************************************************************************************
    func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        // films disponibles?
        if let films = tableauDesFilms["movies"] as? NSArray
        {
            return films.count
        }
        else
        {
            return 0
        }
    } // numberOfItemsInSection
    
    // *************************************************************************************************
    func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell
    // *******************************************************
    //func collectionView(collectionView: UICollectionView!, cellForItemAtIndexPath indexPath: NSIndexPath!) -> UICollectionViewCell!
    
    {
         var celluleCourante = collectionView.dequeueReusableCellWithReuseIdentifier("modeleCellule", forIndexPath: indexPath) as CelluleRotten

        /// *** Temporaire: À effacer!
        celluleCourante.titre!.text = "Cellule no: \(indexPath.row)"
        
        // Cette opération est très bloquante!!!
        // Charger une image à partir du web
        // À compléter ...
        // Renseigner le titre du film
        // À compléter ...
        
        
        // Vérifier s'il la clé 'movie' est présente dans le dictionnaire.
        if let films = tableauDesFilms["movies"] as? NSArray
        {
            celluleCourante.titre.text = (films[indexPath.row] as NSDictionary)["title"] as NSString
            var urlImage = ((films[indexPath.row] as NSDictionary)["posters"] as NSDictionary)["original"] as NSString
            
            // Patch ajoutée 2014.08.25 - rotten ne fournit plus de grosses images dans le feed jason
            urlImage = urlImage.stringByReplacingOccurrencesOfString("tmb", withString: "det")
            celluleCourante.filmImage.image = UIImage(data:NSData.dataWithContentsOfURL(NSURL(string: urlImage), options: nil, error: nil))
        }
        
             return celluleCourante;
    } // cellForItemAtIndexPath


    // *************************************************************************************************
    func rechercherFilms(film:String) -> Bool {
        println("rechercherFilms: \(film)")
        
        progression.startAnimating()
        let adresseURL = "http://api.rottentomatoes.com/api/public/v1.0/movies.json?apikey=\(CLE_API)&q=\(film)&page_limit=\(NB_FILMS)"
        let url: NSURL = NSURL(string: adresseURL)
        
        // Obtenir les données en format brut
        let data = NSData.dataWithContentsOfURL(url, options: nil, error: nil)
        
        // Si data est vide alors quitter la méthode
        if let resultat = data {
            println("L'URL a retourné des données!")
        } else
        {
            println("Erreur: l'URL n'a pas retourné de données!")
            return false
        }
        
        // Convertir les données brutes en format JSON et les placer
        // dans tableauDesFilms
        var error:NSError?
        self.tableauDesFilms = NSJSONSerialization.JSONObjectWithData(data, options: NSJSONReadingOptions.MutableContainers, error: &error) as NSDictionary
        
        // Vérifier si la conversion a fonctionné
        if let err = error {
            // Pour arriver ici, il faut que error != nil, donc de type NSError
            println("Pour une raison étrange, la requête chez 'rottentomatoes' a retournée une structure JSON mal formée...: \(error)");
            return false
        } else
        {
            // error est 'nil' donc il n'y a pas d'erreur!
            println("Conversion JSON réussie");
        }
        
        // println("tableauDesFilms = \(self.tableauDesFilms)")
        
        // Afficher les titres
        for courant in tableauDesFilms["movies"] as [Dictionary<String, AnyObject>] {
            if let titre = courant["title"] as? NSString {
                print("Titre du film: \(titre)")
            }
            if let annee = courant["year"] as? NSNumber {
                println(" - date:\(annee)")
            }
        } // for ... in films

        progression.stopAnimating()
        return true
        
    } // rechercherFilms
    

    // *************************************************************************************************
    func textFieldDidBeginEditing(textField: UITextField) {
        println("textFieldDidBeginEditing")
        progression.startAnimating()
    }  // textFieldDidBeginEditing
    

    // *************************************************************************************************
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        println("textFieldShouldReturn")
       
        /* *********************************
        À compléter ....
        ********************************* */
        // Convertir la chaine en format 'escaped' pour le web, par exemple, remplacer les espaces par %20, ...
        let escapedText = textField.text.stringByAddingPercentEscapesUsingEncoding(NSUTF8StringEncoding)!
        
        println("escapedText = \(escapedText)")
        
        // Relancer la requete vers rottenTomatoes
        rechercherFilms(escapedText)
        // Actualiser les cellules du UICollectionView
        collectionDeFilms.reloadData()
        
        
        //----------------------------------
        textField.resignFirstResponder()
        progression.stopAnimating()
        return true
    } // textFieldShouldReturn
  
    
}  // Fin de la classe ViewController

